@@ -23,7 +23,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H /* sometimes (windows, for instance) it's missing */
+#  include <unistd.h>
+#endif
 #include <errno.h>
 
 #include "barcode.h"
@@ -82,8 +84,17 @@ extern int Barcode_128c_verify(unsigned char *text);
 extern int Barcode_128c_encode(struct Barcode_Item *bc);
 extern int Barcode_128_verify(unsigned char *text);
 extern int Barcode_128_encode(struct Barcode_Item *bc);
+extern int Barcode_128raw_verify(unsigned char *text);
+extern int Barcode_128raw_encode(struct Barcode_Item *bc);
 extern int Barcode_i25_verify(unsigned char *text);
 extern int Barcode_i25_encode(struct Barcode_Item *bc);
+extern int Barcode_cbr_verify(unsigned char *text);
+extern int Barcode_cbr_encode(struct Barcode_Item *bc);
+extern int Barcode_msi_verify(unsigned char *text);
+extern int Barcode_msi_encode(struct Barcode_Item *bc);
+extern int Barcode_pls_verify(unsigned char *text);
+extern int Barcode_pls_encode(struct Barcode_Item *bc);
+
 
 struct encoding {
     int type;
@@ -92,15 +103,19 @@ struct encoding {
 };
 
 struct encoding encodings[] = {
-    {BARCODE_EAN,  Barcode_ean_verify,  Barcode_ean_encode},
-    {BARCODE_UPC,  Barcode_upc_verify,  Barcode_upc_encode},
-    {BARCODE_ISBN, Barcode_isbn_verify, Barcode_isbn_encode},
-    {BARCODE_128B, Barcode_128b_verify, Barcode_128b_encode},
-    {BARCODE_128C, Barcode_128c_verify, Barcode_128c_encode},
-    {BARCODE_39,   Barcode_39_verify,   Barcode_39_encode},
-    {BARCODE_I25,  Barcode_i25_verify,  Barcode_i25_encode},
-    {BARCODE_128,  Barcode_128_verify,  Barcode_128_encode},
-    {0,            NULL,                NULL}
+    {BARCODE_EAN,    Barcode_ean_verify,    Barcode_ean_encode},
+    {BARCODE_UPC,    Barcode_upc_verify,    Barcode_upc_encode},
+    {BARCODE_ISBN,   Barcode_isbn_verify,   Barcode_isbn_encode},
+    {BARCODE_128B,   Barcode_128b_verify,   Barcode_128b_encode},
+    {BARCODE_128C,   Barcode_128c_verify,   Barcode_128c_encode},
+    {BARCODE_128RAW, Barcode_128raw_verify, Barcode_128raw_encode},
+    {BARCODE_39,     Barcode_39_verify,     Barcode_39_encode},
+    {BARCODE_I25,    Barcode_i25_verify,    Barcode_i25_encode},
+    {BARCODE_128,    Barcode_128_verify,    Barcode_128_encode},
+    {BARCODE_CBR,    Barcode_cbr_verify,    Barcode_cbr_encode},
+    {BARCODE_PLS,    Barcode_pls_verify,    Barcode_pls_encode},
+    {BARCODE_MSI,    Barcode_msi_verify,    Barcode_msi_encode},
+    {0,              NULL,                  NULL}
 };
 
 /*
